@@ -8,10 +8,11 @@ from functools import (
 )
 from itertools import islice
 
-from typing import Any, Iterable, Iterator, Union
+from typing import Any, Iterable, Iterator, Union, NewType
+from typing import Protocol, runtime_checkable
+
 from functools import singledispatch, partial
 
-from typing import Protocol, runtime_checkable
 
 IteratorItem = Any
 
@@ -48,6 +49,9 @@ class CursorFunc(Protocol):
         """Get the next iterator's item and increment the cursor"""
 
 
+IterType = NewType('IterType', Union[IteratorType, IterableType, CursorFunc])
+IterType.__doc__ = 'A type that can be made into an iterator'
+
 wrapper_assignments = (*WRAPPER_ASSIGNMENTS, '__defaults__', '__kwdefaults__')
 update_wrapper = partial(_update_wrapper, assigned=wrapper_assignments)
 wraps = partial(_wraps, assigned=wrapper_assignments)
@@ -56,20 +60,56 @@ wraps = partial(_wraps, assigned=wrapper_assignments)
 # ---------------------------------------------------------------------------------------
 # iteratable, iterator, cursors
 no_sentinel = type('no_sentinel', (), {})()
+no_default = type('no_default', (), {})()
 
 
-def iterable_to_iterator(iterable: Iterable) -> Iterator:
+class IteratorExit(BaseException):
+    """Raised when an iterator should quit being iterated on, signaling this event
+    any process that cares to catch the signal.
+    We chose to inherit directly from `BaseException` instead of `Exception`
+    for the same reason that `GeneratorExit` does: Because it's not technically
+    an error.
+
+    See: https://docs.python.org/3/library/exceptions.html#GeneratorExit
+    """
+
+
+DFLT_INTERRUPT_EXCEPTIONS = (StopIteration, IteratorExit, KeyboardInterrupt)
+
+
+def iterate_until_exception(iterator, interrupt_exceptions=DFLT_INTERRUPT_EXCEPTIONS):
+    while True:
+        try:
+            next(iterator)
+        except interrupt_exceptions:
+            print('ending')
+            break
+
+
+def iterable_to_iterator(iterable: Iterable, sentinel=no_sentinel) -> Iterator:
     """Get an iterator from an iterable
 
     >>> iterable = [1, 2, 3]
     >>> iterator = iterable_to_iterator(iterable)
     >>> assert isinstance(iterator, Iterator)
     >>> assert list(iterator) == iterable
+
+    You can also specify a sentinel, which will result in the iterator stoping just
+    before it encounters that sentinel value
+
+    >>> iterable = [1, 2, 3, 4, None, None, 7]
+    >>> iterator = iterable_to_iterator(iterable, None)
+    >>> assert isinstance(iterator, Iterator)
+    >>> list(iterator)
+    [1, 2, 3, 4]
     """
-    return iter(iterable)
+    if sentinel is no_sentinel:
+        return iter(iterable)
+    else:
+        return iter(iter(iterable).__next__, sentinel)
 
 
-def iterator_to_cursor(iterator: Iterator) -> CursorFunc:
+def iterator_to_cursor(iterator: Iterator, default=no_default) -> CursorFunc:
     """Get a cursor function for the input iterator.
 
     >>> iterator = iter([1, 2, 3])
@@ -81,8 +121,31 @@ def iterator_to_cursor(iterator: Iterator) -> CursorFunc:
     Note how we consumed the cursor till the end; by using cursor_to_iterator.
     Indeed, `list(iter(cursor))` wouldn't have worked since a cursor isn't a iterator,
     but a callable to get the items an the iterator would give you.
+
+    You can specify a default. The default has the same role that it has for the
+    `next` function: It makes the cursor function return that default when the iterator
+    has been "consumed" (i.e. would raise a `StopIteration`).
+
+    >>> iterator = iter([1, 2])
+    >>> cursor = iterator_to_cursor(iterator, None)
+    >>> assert callable(cursor)
+    >>> cursor()
+    1
+    >>> cursor()
+    2
+
+    And then...
+
+    >>> assert cursor() is None
+    >>> assert cursor() is None
+
+    forever.
+
     """
-    return partial(next, iterator)
+    if default is no_default:
+        return partial(next, iterator)
+    else:
+        return partial(next, iterator, default)
 
 
 def cursor_to_iterator(cursor: CursorFunc, sentinel=no_sentinel) -> Iterator:
@@ -233,3 +296,36 @@ static_identity_method = staticmethod(identity_func)
 # @iterable_to_iterator.register
 # def _(iterable: AsyncIterable) -> AsyncIterator:
 #     return aiter(iterable)
+class Pipe:
+    """Simple function composition. That is, gives you a callable that implements input -> f_1 -> ... -> f_n -> output.
+
+    >>> def foo(a, b=2):
+    ...     return a + b
+    >>> f = Pipe(foo, lambda x: print(f"x: {x}"))
+    >>> f(3)
+    x: 5
+
+    Notes:
+        - Pipe instances don't have a __name__ etc. So some expectations of normal functions are not met.
+        - Pipe instance are pickalable (as long as the functions that compose them are)
+    """
+
+    def __init__(self, *funcs):
+
+        n_funcs = len(funcs)
+        other_funcs = ()
+        if n_funcs == 0:
+            raise ValueError('You need to specify at least one function!')
+        elif n_funcs == 1:
+            first_func = last_func = funcs[0]
+        else:
+            first_func, *other_funcs, last_func = funcs
+
+        self.first_func = first_func
+        self.other_funcs = tuple(other_funcs) + (last_func,)
+
+    def __call__(self, *args, **kwargs):
+        out = self.first_func(*args, **kwargs)
+        for func in self.other_funcs:
+            out = func(out)
+        return out
